// carddeck0.c
// Chapter 16
// Learn C Programming, 2nd Edition
//
// Starting point (from Chapter 10, card4.c) for complex structures.
// Modify it to reflect data structures and operations using arrays and 
// pointers.
//
// In carddeck_0.c, remove unneeded functions (we'll add them back later)
// and demonstrate an array of structures.
//
// Also: 
//  1. add GetCardFaceValue() and GetCardSuitValue()
//  2. use these functions in InitializeCard()
//  3. enum names
//
// compile with 
//   cc carddeck_0.c -o carddeck_0 -Wall -Werror =std=c11


#include <stdio.h>
#include <stdbool.h>
#include <string.h>


  // ============================================
  // Definitions for a Card
  // ============================================

  // Card Suits
typedef enum {
  eClub  = 1, 
  eDiamond, 
  eHeart,
  eSpade 
} Suit;

  // Card Faces
typedef enum  {
  eOne = 1,
  eTwo ,
  eThree ,
  eFour ,
  eFive ,
  eSix ,
  eSeven ,
  eEight ,
  eNine ,
  eTen ,
  eJack ,
  eQueen ,
  eKing ,
  eAce
} Face;

  // Card
typedef struct  {
 Suit suit;
 int  suitValue;
 Face face;
 int  faceValue;
 bool isWild;
} Card;


  // Operations on a Card
void InitializeCard(   Card* pCard , Suit s , Face f , bool w );
void PrintCard(        Card* pCard );
void CardToString(     Card* pCard , char pCardStr[20] );
int  GetCardFaceValue( Card* pCard );
int  GetCardSuitValue( Card* pCard );


int main( void )  {
  Card aCard;
  InitializeCard( &aCard, eDiamond , eSeven , true );
  PrintCard(      &aCard );
  printf( "\n" );
}


  // ============================================
  // Operations on a Card
  // ============================================


void InitializeCard( Card* pCard, Suit s , Face f , bool w ) {
  pCard->suit = s; 
  pCard->suitValue = GetCardSuitValue( pCard );
  
  pCard->face = f;
  pCard->faceValue = GetCardFaceValue( pCard );
  
  pCard->isWild = w;
}


void PrintCard( Card* pCard )  {
  char cardStr[20] = {0};
  CardToString( pCard , cardStr );
  printf( "%18s" , cardStr );
}


void CardToString( Card* pCard , char pCardStr[20] )  {
  switch( pCard->face ) {
    case eTwo:   strcpy( pCardStr , "    2 " ); break;
    case eThree: strcpy( pCardStr , "    3 " ); break;
    case eFour:  strcpy( pCardStr , "    4 " ); break;
    case eFive:  strcpy( pCardStr , "    5 " ); break;
    case eSix:   strcpy( pCardStr , "    6 " ); break;
    case eSeven: strcpy( pCardStr , "    7 " ); break;
    case eEight: strcpy( pCardStr , "    8 " ); break;
    case eNine:  strcpy( pCardStr , "    9 " ); break;
    case eTen:   strcpy( pCardStr , "   10 " ); break;
    case eJack:  strcpy( pCardStr , " Jack " ); break;
    case eQueen: strcpy( pCardStr , "Queen " ); break;
    case eKing:  strcpy( pCardStr , " King " ); break;
    case eAce:   strcpy( pCardStr , "  Ace " ); break;
    default:     strcpy( pCardStr , "  ??? " ); break;
  }
  switch( pCard->suit )  {
    case eSpade:   strcat( pCardStr , "of Spades  "); break;
    case eHeart:   strcat( pCardStr , "of Hearts  "); break;
    case eDiamond: strcat( pCardStr , "of Diamonds"); break;
    case eClub:    strcat( pCardStr , "of Clubs   "); break;
    default:       strcat( pCardStr , "of ???s    "); break;
  }
}


  // For now, rely upon proper definition of enum Faces.
  // If, at some future time, the face values need to chage,
  // this function can be changed as needed and program will continue
  // to work as expected.
inline int GetCardFaceValue( Card* pCard )  {
  return (int)pCard->face;
}


  // For now, realy upon proper definition of enum Suits.
  // If, at some future time, the suit values need to chage,
  // this function can be changed as needed and program will continue
  // to work as expected.
inline int GetCardSuitValue( Card* pCard ) {
  return (int)pCard->suit;
}


  /* eof */
  